"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var jsx_runtime_1 = require("@emotion/react/jsx-runtime");
var react_1 = __importDefault(require("react"));
var style_1 = __importDefault(require("./style"));
var error_stack_parser_1 = __importDefault(require("error-stack-parser"));
var helpers_1 = require("./helpers");
var sourcemapped_stacktrace_1 = require("sourcemapped-stacktrace");
var redbox = style_1.default.redbox, message = style_1.default.message, stack = style_1.default.stack, frame = style_1.default.frame, file = style_1.default.file, linkToFile = style_1.default.linkToFile;
var RedBoxFrames = function (_a) {
    var stack = _a.stack, filename = _a.filename, editorScheme = _a.editorScheme, _b = _a.useLines, useLines = _b === void 0 ? true : _b, _c = _a.useColumns, useColumns = _c === void 0 ? true : _c;
    var frames = react_1.default.useMemo(function () {
        if (stack) {
            try {
                return error_stack_parser_1.default.parse({ stack: stack }).map(function (_a, index) {
                    var fileName = _a.fileName, functionName = _a.functionName, lineNumber = _a.lineNumber, columnNumber = _a.columnNumber;
                    var text;
                    var url;
                    if (index === 0 && filename && !(0, helpers_1.isFilenameAbsolute)(fileName)) {
                        url = (0, helpers_1.makeUrl)(filename, editorScheme);
                        text = (0, helpers_1.makeLinkText)(filename);
                    }
                    else {
                        var lines = useLines ? lineNumber : undefined;
                        var columns = useColumns ? columnNumber : undefined;
                        url = (0, helpers_1.makeUrl)(fileName, editorScheme, lines, columns);
                        text = (0, helpers_1.makeLinkText)(fileName, lines, columns);
                    }
                    return {
                        functionName: functionName,
                        url: url,
                        text: text,
                    };
                });
            }
            catch (e) {
                return undefined;
            }
        }
    }, [stack]);
    return ((0, jsx_runtime_1.jsx)(jsx_runtime_1.Fragment, { children: frames === null || frames === void 0 ? void 0 : frames.map(function (_a, index) {
            var functionName = _a.functionName, url = _a.url, text = _a.text;
            return ((0, jsx_runtime_1.jsxs)("div", __assign({ style: frame }, { children: [(0, jsx_runtime_1.jsx)("div", { children: functionName }), (0, jsx_runtime_1.jsx)("div", __assign({ style: file }, { children: (0, jsx_runtime_1.jsx)("a", __assign({ href: url, style: linkToFile }, { children: text })) }))] }), index));
        }) }));
};
var MemoisedFrames = react_1.default.memo(RedBoxFrames);
var RedBox = function (_a) {
    var error = _a.error, filename = _a.filename, context = _a.context, _b = _a.showStackTrace, showStackTrace = _b === void 0 ? true : _b, _c = _a.editorScheme, editorScheme = _c === void 0 ? "file" : _c, _d = _a.useLines, useLines = _d === void 0 ? true : _d, _e = _a.useColumns, useColumns = _e === void 0 ? true : _e;
    var _f = react_1.default.useState(), mappedStack = _f[0], setMappedStack = _f[1];
    react_1.default.useEffect(function () {
        var cancelled = false;
        setMappedStack(undefined);
        if (!error.stack) {
            return;
        }
        var stackLines = error.stack.split("\n");
        // There's no stack, only the error message.
        if (!stackLines || stackLines.length < 2) {
            setMappedStack(error.stack);
        }
        // Using the “eval” setting on webpack already gives the correct location.
        var isWebpackEval = stackLines[1].search(/\(webpack:\/{3}/) !== -1;
        if (isWebpackEval) {
            // No changes are needed here.
            setMappedStack(error.stack);
        }
        // Other eval follow a specific pattern and can be easily parsed.
        var isEval = stackLines[1].search(/\(eval at/) !== -1;
        if (!isEval) {
            (0, sourcemapped_stacktrace_1.mapStackTrace)(error.stack, function (mappedStack) {
                if (!cancelled) {
                    setMappedStack(mappedStack.join("\n"));
                }
            });
            return function () {
                cancelled = true;
            };
        }
        // The first line is the error message.
        var fixedLines = [stackLines.shift()];
        // The rest needs to be fixed.
        for (var _i = 0, stackLines_1 = stackLines; _i < stackLines_1.length; _i++) {
            var stackLine = stackLines_1[_i];
            var evalStackLine = stackLine.match(/(.+)\(eval at (.+) \(.+?\), .+(:[0-9]+:[0-9]+)\)/);
            if (evalStackLine) {
                var atSomething = evalStackLine[1], file_1 = evalStackLine[2], rowColumn = evalStackLine[3];
                fixedLines.push("".concat(atSomething, " (").concat(file_1).concat(rowColumn, ")"));
            }
            else {
                // TODO: When stack frames of different types are detected, try to load the additional source maps
                fixedLines.push(stackLine);
            }
        }
        setMappedStack(fixedLines.join("\n"));
    }, [error]);
    return ((0, jsx_runtime_1.jsx)(jsx_runtime_1.Fragment, { children: error && ((0, jsx_runtime_1.jsxs)("div", __assign({ style: redbox }, { children: [(0, jsx_runtime_1.jsxs)("div", __assign({ style: message }, { children: [error.name, ": ", error.message, context && ((0, jsx_runtime_1.jsxs)(jsx_runtime_1.Fragment, { children: [(0, jsx_runtime_1.jsx)("br", {}), (0, jsx_runtime_1.jsx)("br", {}), context] }))] })), showStackTrace && mappedStack && ((0, jsx_runtime_1.jsx)("div", __assign({ style: stack }, { children: (0, jsx_runtime_1.jsx)(MemoisedFrames, { stack: mappedStack, filename: filename, editorScheme: editorScheme, useLines: useLines, useColumns: useColumns }) })))] }))) }));
};
exports.default = RedBox;
