"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeLinkText = exports.makeUrl = exports.isFilenameAbsolute = exports.filenameHasSchema = exports.filenameHasLoaders = exports.filenameWithoutLoaders = void 0;
var filenameWithoutLoaders = function (filename) {
    if (filename === void 0) { filename = ""; }
    var index = filename.lastIndexOf("!");
    return index < 0 ? filename : filename.substring(index + 1);
};
exports.filenameWithoutLoaders = filenameWithoutLoaders;
var filenameHasLoaders = function (filename) {
    var actualFilename = (0, exports.filenameWithoutLoaders)(filename);
    return actualFilename !== filename;
};
exports.filenameHasLoaders = filenameHasLoaders;
var filenameHasSchema = function (filename) {
    return filename ? /^[\w]+:/.test(filename) : false;
};
exports.filenameHasSchema = filenameHasSchema;
var isFilenameAbsolute = function (filename) {
    var actualFilename = (0, exports.filenameWithoutLoaders)(filename);
    if (actualFilename.indexOf("/") === 0) {
        return true;
    }
    return false;
};
exports.isFilenameAbsolute = isFilenameAbsolute;
var makeUrl = function (filename, scheme, line, column) {
    var actualFilename = (0, exports.filenameWithoutLoaders)(filename);
    if ((0, exports.filenameHasSchema)(filename)) {
        return actualFilename;
    }
    var url = "file://".concat(actualFilename);
    if (scheme === "vscode") {
        url = "".concat(scheme, "://file/").concat(url);
        url = url.replace(/file:\/\/\//, ""); // visual studio code does not need file:/// in its scheme
        if (line && actualFilename === filename) {
            url = "".concat(url, ":").concat(line);
            if (column) {
                url = "".concat(url, ":").concat(column);
            }
        }
    }
    else if (scheme) {
        url = "".concat(scheme, "://open?url=").concat(url);
        if (line && actualFilename === filename) {
            url = "".concat(url, "&line=").concat(line);
            if (column) {
                url = "".concat(url, "&column=").concat(column);
            }
        }
    }
    return url;
};
exports.makeUrl = makeUrl;
var makeLinkText = function (filename, line, column) {
    var text = (0, exports.filenameWithoutLoaders)(filename);
    if (line && text === filename) {
        text = "".concat(text, ":").concat(line);
        if (column) {
            text = "".concat(text, ":").concat(column);
        }
    }
    return text;
};
exports.makeLinkText = makeLinkText;
