"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
function download(content, name, mimetype) {
    var a = document.createElement("a");
    a.href = URL.createObjectURL(new Blob([content], { type: mimetype }));
    a.download = name;
    a.style.display = "none";
    document.body.appendChild(a);
    a.click();
    URL.revokeObjectURL(a.href);
    document.body.removeChild(a);
}
function useExport(core, addTask, node, name) {
    var linearDeflection = 0.05;
    var angularDeflection = 0.5;
    var exportBREP = react_1.default.useCallback(function () {
        return addTask(function () {
            var progress = core.renderBREP(node);
            progress.then(function (content) {
                return content &&
                    download(content, "".concat(name || "react-cad", ".brep"), "model/brep");
            });
            return progress;
        }, false);
    }, [core, addTask, node, name]);
    var exportSTEP = react_1.default.useCallback(function () {
        return addTask(function () {
            var progress = core.renderSTEP(node);
            progress.then(function (content) {
                return content &&
                    download(content, "".concat(name || "react-cad", ".step"), "model/step");
            });
            return progress;
        }, false);
    }, [core, addTask, node, name]);
    var exportSTL = react_1.default.useCallback(function () {
        return addTask(function () {
            var progress = core.renderSTL(node, linearDeflection, angularDeflection);
            progress.then(function (content) {
                return content &&
                    download(content, "".concat(name || "react-cad", ".stl"), "model/stl");
            });
            return progress;
        }, false);
    }, [core, node, addTask, name]);
    return { exportSTL: exportSTL, exportBREP: exportBREP, exportSTEP: exportSTEP };
}
exports.default = useExport;
